@@ -1,7 +1,7 @@
 # policy.py - module for handling PSKC policy information
 # coding: utf-8
 #
-# Copyright (C) 2014-2015 Arthur de Jong
+# Copyright (C) 2014-2016 Arthur de Jong
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -21,6 +21,14 @@
 """Module that provides PSKC key policy information."""
 
 
+def _make_aware(d):
+    """Make tge specified datetime timezone aware."""
+    import dateutil.tz
+    if not d.tzinfo:
+        return d.replace(tzinfo=dateutil.tz.tzlocal())
+    return d
+
+
 class Policy(object):
     """Representation of a policy that describes key and pin usage.
 
@@ -114,14 +122,14 @@ class Policy(object):
         if policy is None:
             return
 
-        self.start_date = findtime(policy, 'pskc:StartDate')
-        self.expiry_date = findtime(policy, 'pskc:ExpiryDate')
+        self.start_date = findtime(policy, 'StartDate')
+        self.expiry_date = findtime(policy, 'ExpiryDate')
         self.number_of_transactions = findint(
-            policy, 'pskc:NumberOfTransactions')
-        for key_usage in findall(policy, 'pskc:KeyUsage'):
+            policy, 'NumberOfTransactions')
+        for key_usage in findall(policy, 'KeyUsage'):
             self.key_usage.append(findtext(key_usage, '.'))
 
-        pin_policy = find(policy, 'pskc:PINPolicy')
+        pin_policy = find(policy, 'PINPolicy')
         if pin_policy is not None:
             self.pin_key_id = pin_policy.get('PINKeyId')
             self.pin_usage = pin_policy.get('PINUsageMode')
@@ -130,11 +138,23 @@ class Policy(object):
             self.pin_min_length = getint(pin_policy, 'MinLength')
             self.pin_max_length = getint(pin_policy, 'MaxLength')
             self.pin_encoding = pin_policy.get('PINEncoding')
-            # TODO: check if there are any other attributes set for PINPolicy
-            # of if there are any children and set unknown_policy_elementss
+            # check for child elements
+            if list(pin_policy):
+                self.unknown_policy_elements = True
+            # check for unknown attributes
+            known_attributes = set([
+                'PINKeyId', 'PINUsageMode', 'MaxFailedAttempts', 'MinLength',
+                'MaxLength', 'PINEncoding'])
+            if set(pin_policy.keys()) - known_attributes:
+                self.unknown_policy_elements = True
 
-        # TODO: check if there are other children and make sure
-        # policy rejects any key usage (set unknown_policy_elements)
+        # check for other child elements
+        known_children = set([
+            'StartDate', 'ExpiryDate', 'NumberOfTransactions', 'KeyUsage',
+            'PINPolicy'])
+        for child in policy:
+            if child.tag not in known_children:
+                self.unknown_policy_elements = True
 
     def make_xml(self, key):
         from pskc.xml import mk_elem
@@ -145,8 +165,6 @@ class Policy(object):
                 self.pin_max_failed_attemtps, self.pin_min_length,
                 self.pin_max_length, self.pin_encoding)):
             return
-        # TODO: raise exception if unknown_policy_elements is set
-
         policy = mk_elem(key, 'pskc:Policy', empty=True)
         mk_elem(policy, 'pskc:StartDate', self.start_date)
         mk_elem(policy, 'pskc:ExpiryDate', self.expiry_date)
@@ -162,11 +180,25 @@ class Policy(object):
         mk_elem(policy, 'pskc:NumberOfTransactions',
                 self.number_of_transactions)
 
-    def may_use(self, usage):
+    def may_use(self, usage=None, now=None):
         """Check whether the key may be used for the provided purpose."""
+        import datetime
+        import dateutil.tz
         if self.unknown_policy_elements:
             return False
-        return not self.key_usage or usage in self.key_usage
+        if usage is not None and self.key_usage:
+            if usage not in self.key_usage:
+                return False
+        # check start_date and expiry_date
+        if now is None:
+            now = datetime.datetime.now(dateutil.tz.tzlocal())
+        if self.start_date:
+            if _make_aware(self.start_date) > _make_aware(now):
+                return False  # not-yet usable key
+        if self.expiry_date:
+            if _make_aware(self.expiry_date) < _make_aware(now):
+                return False  # not-yet usable key
+        return True
 
     @property
     def pin_key(self):
